﻿/* 
	The scripts below assumes you created the table si_streets as shown
	in the slides (Lectures 7+8).
	
	Uncomment and execute each segment seperately to see the results. 
	Some of the processing is quite heavy and might take some time 
	depending on your machine 
*/

/*********************
First make the data routable: I do this by breaking every linestring down into line segments
*********************/
-- DROP TABLE IF EXISTS street_segments;
-- 
-- WITH RECURSIVE segments(gid, id, name, oneway, type, sp, ep,sid) AS (
-- 	SELECT
-- 		gid, id, name, oneway, type, 
-- 		ST_PointN(geom, 1) as sp,
-- 		ST_PointN(geom, 2) as ep,
-- 		1 as sid
-- 	FROM
-- 	-- extract the individual linestrings
-- 		(SELECT  gid, id, name, oneway, type, ST_GeometryN(geom, 1) AS geom FROM si_streets) AS ls1
--    
-- 	UNION ALL
-- 		SELECT
-- 			ls2.gid, ls2.id, ls2.name, ls2.oneway, ls2.type, 
-- 			ST_PointN(ls2.geom, ss.sid+1) as sp,
-- 			ST_PointN(ls2.geom, ss.sid+2) as ep,
-- 			ss.sid+1 AS sid
-- 		FROM
-- 		-- extract the individual linestrings
-- 		(SELECT  gid, id, name, oneway, type, ST_GeometryN(geom, 1) AS geom
-- 			FROM si_streets
-- 		) AS ls2, segments AS ss WHERE ls2.gid = ss.gid AND ss.sid < ST_NPoints(ls2.geom)-1
-- )
-- SELECT gid, sid, id, name, oneway, type, ST_MakeLine(sp,ep) AS geom INTO street_segments FROM segments ORDER BY gid, sid;
-- 
-- SELECT gid, sid, id, name, oneway, type, ST_AsText(geom) FROM street_segments ORDER BY gid, sid;

/*********************
Then make each segment unique by creating the unique id sgid. We need this for pgRouting
*********************/
-- ALTER TABLE street_segments ADD COLUMN sgid integer;

-- UPDATE street_segments SET sgid = (gid * (SELECT max(g.cnt) FROM (SELECT count(gid) AS cnt FROM street_segments GROUP BY gid) AS g)) + sid;

/*********************
Then we can start following the stuff in the slides. 
Note that we must replace the old table si_streets with 
the table street_segments
*********************/
-- ALTER TABLE street_segments ADD COLUMN "source" integer;
-- ALTER TABLE street_segments ADD COLUMN "target" integer;
-- SELECT pgr_createTopology('street_segments', 0.00001, 'geom', 'sgid');

/*********************
Ignore this part. Skip to the next lines
*********************/
-- CREATE INDEX street_segments_source_idx ON street_segments("source");
-- CREATE INDEX street_segments_target_idx ON street_segments("target");


/*********************
Add and update the cost columns
*********************/
-- ALTER TABLE street_segments ADD COLUMN "cost" double precision;
-- ALTER TABLE street_segments ADD COLUMN "reverse_cost" double precision;

-- UPDATE street_segments SET cost = ST_Length(geom), reverse_cost = ST_Length(geom);

-- SELECT sgid, gid, sid, id, name, oneway, type, ST_AsText(geom) FROM street_segments ORDER BY sgid;

-- SELECT * FROM street_segments ORDER BY sgid;

/*********************
Now try the routing
*********************/
-- SELECT seq, id1 AS node, id2 AS edge, cost FROM
-- pgr_dijkstra(
-- 	'SELECT sgid AS id, source::integer, target::integer, cost::double precision FROM street_segments', 
-- 	15054, 11530, false, false
-- );

/*********************
Routing with geometry output saved to table. You can use this in QGIS to 
immediately visualize the result.
*********************/
-- SELECT seq, id1 AS node, id2 AS edge, rt.cost,
-- si.geom INTO route FROM pgr_dijkstra('
--  SELECT sgid AS id,
--  source::integer,
--  target::integer,
--  cost::double precision
--  FROM street_segments',
-- 15054, 11530, false, false
-- ) rt LEFT JOIN street_segments si ON (rt.id2 = si.sgid);
-- 
-- SELECT * FROM route;